c========================================================================
      PROGRAM FLUX_MODEL
c========================================================================
c
c    AUTHOR: Michael H. Denton
c    Copyright 2015,2016 Michael H. Denton
c    Version 1.0 : 13/06/2016
c
c    GNU-GPL v3 (see below and file gpl.txt, which accompanies this code)
c
c    A full summary of the model can be found in the publication that 
c    accompanies this code:
c
c    "An empirical model of electron and ion fluxes derived from
c    observations at geosynchronous orbit",  
c    M. H. Denton, M. F. Thomsen, V. K. Jordanova, M. G. Henderson,
c    J. E. Borovsky, J. S. Denton, D. Pitchford, and D. P. Hartley.
c    Space Weather, 10.1002/2015SW001168, 2015.
c
c    "An improved empirical model of electron and ion fluxes at 
c    geosynchronous orbit based on upstream solar wind conditions",
c    M. H. Denton, M. G. Henderson, V. K. Jordanova, M. F. Thomsen, 
c    J. E. Borovsky, J. Woodroffe, D. P. Hartley, and D. Pitchford, 2016.
c    (under review)
c
c    Help and suggestions of co-authors is gratefully acknowledged!!!
c
c    NOTE: Flux values here are more precise than quoted in the paper 
c    due to slight improvements in the satellite centre-energies.     
c
c    If you find the code useful, or have suggestions for changes, then 
c    please let me know.  Mick Denton - mdenton@spacescience.org
c
c=============================PROGRAM OVERVIEW===========================
c
c    The code returns flux values for GEOSYNCHRONOUS ORBIT as a function
c    of energy, local-time, and Kp index.  The underlying data for 
c    the model is from the Los Alamos National Laboratory MPA 
c    instrument (PI M. G. Henderson).
c
c    Model bounds: ~1 eV to ~40 keV, any LT, any Kp.
c
c    CAVEAT: Electron Fluxes are only reliable at E >~100 eV
c            Below 100 eV there may be contamination by photoelectrons
c            and secondary electrons.
c
c    INPUTS:
c    The user may enter the following:
c
c      choose_en:  Energy in eV, from ~1ev to ~40000 keV.  
c                  e.g. 25500=25.5 keV.
c
c      choose_kp:  Kp (1-28) for one of the 28 values of Kp from 0 to 9
c                  e.g. choose_kp=5, actual_Kp is 1+
c
c      choose_lt: (e.g. 12.80 LT)
c
c    OUTPUTS: 
c    The model returns the following:
c
c  Mean electron flux: (ALL F10.7, F10.7<100, 100<=F10.7<170, F10.7=>170) 
c  Standard deviation on the mean electron flux
c   5th percentile electron flux (five)
c  25th percentile electron flux (tfive)
c  50th percentile electron flux (med)
c  75th percentile electron flux (sfive)
c  95th percentile electron flux (nfive)
c 
c  Mean ion flux : (ALL F10.7, F10.7<100, 100<=F10.7<170, F10.7=>170)
c  Standard deviation on the mean ion flux
c   5th percentile ion flux (five)
c  25th percentile ion flux (tfive)
c  50th percentile ion flux (med)
c  75th percentile ion flux (sfive)
c  95th percentile ion flux (nfive)
c    
c  Fluxes are given in units of (cm^-2 s^-1 sr^-1 eV^-1)
c
c    DESCRIPTION OF SUBROUTINES
c  
c    flux_model.f: 
c          The MAIN subroutine.
c    bilinear.f: 
c          Function which performs bilinear interpolation.
c    calc.f:
c          Subroutine which takes the input and returns flux values.
c    data_read.f:
c          Subroutine to read in the underlying MPA data files.
c
c    GNU GENERAL PUBLIC LICENSE
c    This program is free software: you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation, either version 3 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details (file gpl.txt).
c
c    You should have received a copy of the GNU General Public License
c    along with this program.  If not, see <http://www.gnu.org/licenses/>
c
c    Please feel free to modify the code to suit your own needs in line 
c    with this license.
c
c========================================================================

      IMPLICIT NONE

c     DEFINE INTEGER VARIABLES
      INTEGER(KIND=8) KPS, ENS, LTS, VALS
      INTEGER(KIND=8) I, J, K
      INTEGER(KIND=8) choose_kp

c     DEFINE PARAMETERS
      PARAMETER(KPS=28, ENS=40, LTS=24, VALS=7)

c     DEFINE INTEGER ARRAYS
      INTEGER(KIND=8) kp_array(KPS)

c     DEFINE CHARACTER ARRAYS
      CHARACTER str_kp(KPS)*2
      CHARACTER str_arr(VALS)*10
      CHARACTER head_arr(4)*16

c     DEFINE REAL ARRAYS
      REAL(KIND=8) mean_e_flux_all(KPS,ENS,LTS)
      REAL(KIND=8) std_e_flux_all(KPS,ENS,LTS)
      REAL(KIND=8) five_e_flux_all(KPS,ENS,LTS)
      REAL(KIND=8) tfive_e_flux_all(KPS,ENS,LTS)
      REAL(KIND=8) med_e_flux_all(KPS,ENS,LTS)
      REAL(KIND=8) sfive_e_flux_all(KPS,ENS,LTS)
      REAL(KIND=8) nfive_e_flux_all(KPS,ENS,LTS)

      REAL(KIND=8) mean_e_flux_low(KPS,ENS,LTS)
      REAL(KIND=8) std_e_flux_low(KPS,ENS,LTS)
      REAL(KIND=8) five_e_flux_low(KPS,ENS,LTS)
      REAL(KIND=8) tfive_e_flux_low(KPS,ENS,LTS)
      REAL(KIND=8) med_e_flux_low(KPS,ENS,LTS)
      REAL(KIND=8) sfive_e_flux_low(KPS,ENS,LTS)
      REAL(KIND=8) nfive_e_flux_low(KPS,ENS,LTS)

      REAL(KIND=8) mean_e_flux_mid(KPS,ENS,LTS)
      REAL(KIND=8) std_e_flux_mid(KPS,ENS,LTS)
      REAL(KIND=8) five_e_flux_mid(KPS,ENS,LTS)
      REAL(KIND=8) tfive_e_flux_mid(KPS,ENS,LTS)
      REAL(KIND=8) med_e_flux_mid(KPS,ENS,LTS)
      REAL(KIND=8) sfive_e_flux_mid(KPS,ENS,LTS)
      REAL(KIND=8) nfive_e_flux_mid(KPS,ENS,LTS)

      REAL(KIND=8) mean_e_flux_high(KPS,ENS,LTS)
      REAL(KIND=8) std_e_flux_high(KPS,ENS,LTS)
      REAL(KIND=8) five_e_flux_high(KPS,ENS,LTS)
      REAL(KIND=8) tfive_e_flux_high(KPS,ENS,LTS)
      REAL(KIND=8) med_e_flux_high(KPS,ENS,LTS)
      REAL(KIND=8) sfive_e_flux_high(KPS,ENS,LTS)
      REAL(KIND=8) nfive_e_flux_high(KPS,ENS,LTS)

      REAL(KIND=8) mean_i_flux_all(KPS,ENS,LTS)
      REAL(KIND=8) std_i_flux_all(KPS,ENS,LTS)
      REAL(KIND=8) five_i_flux_all(KPS,ENS,LTS)
      REAL(KIND=8) tfive_i_flux_all(KPS,ENS,LTS)
      REAL(KIND=8) med_i_flux_all(KPS,ENS,LTS)
      REAL(KIND=8) sfive_i_flux_all(KPS,ENS,LTS)
      REAL(KIND=8) nfive_i_flux_all(KPS,ENS,LTS)

      REAL(KIND=8) mean_i_flux_low(KPS,ENS,LTS)
      REAL(KIND=8) std_i_flux_low(KPS,ENS,LTS)
      REAL(KIND=8) five_i_flux_low(KPS,ENS,LTS)
      REAL(KIND=8) tfive_i_flux_low(KPS,ENS,LTS)
      REAL(KIND=8) med_i_flux_low(KPS,ENS,LTS)
      REAL(KIND=8) sfive_i_flux_low(KPS,ENS,LTS)
      REAL(KIND=8) nfive_i_flux_low(KPS,ENS,LTS)

      REAL(KIND=8) mean_i_flux_mid(KPS,ENS,LTS)
      REAL(KIND=8) std_i_flux_mid(KPS,ENS,LTS)
      REAL(KIND=8) five_i_flux_mid(KPS,ENS,LTS)
      REAL(KIND=8) tfive_i_flux_mid(KPS,ENS,LTS)
      REAL(KIND=8) med_i_flux_mid(KPS,ENS,LTS)
      REAL(KIND=8) sfive_i_flux_mid(KPS,ENS,LTS)
      REAL(KIND=8) nfive_i_flux_mid(KPS,ENS,LTS)

      REAL(KIND=8) mean_i_flux_high(KPS,ENS,LTS)
      REAL(KIND=8) std_i_flux_high(KPS,ENS,LTS)
      REAL(KIND=8) five_i_flux_high(KPS,ENS,LTS)
      REAL(KIND=8) tfive_i_flux_high(KPS,ENS,LTS)
      REAL(KIND=8) med_i_flux_high(KPS,ENS,LTS)
      REAL(KIND=8) sfive_i_flux_high(KPS,ENS,LTS)
      REAL(KIND=8) nfive_i_flux_high(KPS,ENS,LTS)

      REAL(KIND=8) final_e_flux_all(VALS),  final_i_flux_all(VALS)
      REAL(KIND=8) final_e_flux_low(VALS),  final_i_flux_low(VALS)
      REAL(KIND=8) final_e_flux_mid(VALS),  final_i_flux_mid(VALS)
      REAL(KIND=8) final_e_flux_high(VALS), final_i_flux_high(VALS)

c     DEFINE REAL VARIABLES
      REAL(KIND=8) choose_en, choose_lt

      DATA str_kp/ 
     >             '0', '0+', '1-', '1o', '1+', '2-', '2o', 
     >             '2+','3-', '3o', '3+', '4-', '4o', '4+',
     >             '5-','5o', '5+', '6-', '6o', '6+', '7-',
     >             '7o','7+', '8-', '8o', '8+', '9-', '9o'/

      DATA kp_array/ 
     >               1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,
     >              15,16,17,18,19,20,21,22,23,24,25,26,27,28/

      DATA str_arr/
     >'      MEAN','   STD-DEV','   5TH PER','  25th PER','   MEDIAN',
     >'  75th PER','  95th PER'/

      DATA head_arr/
     >'       ALL F10.7','       100<F10.7',
     >'  100<=F10.7<170','      F10.7=>170'/

c     ===================================================================
c     --------------END DEFINITIONS-BEGIN MODEL--------------------------
c     ===================================================================

c     PRINT HEADER TO SCREEN
      WRITE(6,*) ''
      WRITE(6,*) '====================================================='
      WRITE(6,*) '   AUTHOR:   Michael H. Denton'
      WRITE(6,*) '   CODE:     FLUX_MODEL (Kp Version)'
      WRITE(6,*) '   VERSION:  1.0 (13/06/2016)'
      WRITE(6,*) ''
      WRITE(6,*) '      flux_model.f'
      WRITE(6,*) '        bilinear.f'
      WRITE(6,*) '            calc.f'
      WRITE(6,*) '       data_read.f'
      WRITE(6,*) ''
      WRITE(6,*) '  This is free software under the terms of the GNU '
      WRITE(6,*) '  General Public License v3.  You are welcome to '
      WRITE(6,*) '  redistribute it under certain conditions. You'
      WRITE(6,*) '  should have received a copy of the GNU General'
      WRITE(6,*) '  Public License along with this program. If not, see'
      WRITE(6,*) '  http://gnu.org/licenses'
      WRITE(6,*) ''
      WRITE(6,*) '  Electron energy channels below 100 eV are likely '
      WRITE(6,*) '  by photoelectrons and secondaries!'
      WRITE(6,*) ''
      WRITE(6,*) '  Feel free to modify in accordance with the license!'
      WRITE(6,*) ''
      WRITE(6,*) '  This program comes with ABSOLUTELY NO WARRANTY!!!'
      WRITE(6,*) '====================================================='
      WRITE(6,*) ''

      CALL SLEEP(2)

c     NOW READ IN THE DATA!!!
      CALL DATA_READ(
     >mean_e_flux_all,mean_e_flux_low,mean_e_flux_mid,mean_e_flux_high,
     > std_e_flux_all, std_e_flux_low, std_e_flux_mid, std_e_flux_high,
     >five_e_flux_all,five_e_flux_low,five_e_flux_mid,five_e_flux_high,
     >tfive_e_flux_all,tfive_e_flux_low,tfive_e_flux_mid,
     >tfive_e_flux_high,
     > med_e_flux_all, med_e_flux_low, med_e_flux_mid, med_e_flux_high,
     >sfive_e_flux_all,sfive_e_flux_low,sfive_e_flux_mid,
     >sfive_e_flux_high,
     >nfive_e_flux_all,nfive_e_flux_low,nfive_e_flux_mid,
     >nfive_e_flux_high,
     >mean_i_flux_all,mean_i_flux_low,mean_i_flux_mid,mean_i_flux_high,
     > std_i_flux_all, std_i_flux_low, std_i_flux_mid, std_i_flux_high,
     >five_i_flux_all,five_i_flux_low,five_i_flux_mid,five_i_flux_high,
     >tfive_i_flux_all,tfive_i_flux_low,tfive_i_flux_mid,
     >tfive_i_flux_high,
     > med_i_flux_all, med_i_flux_low, med_i_flux_mid, med_i_flux_high,
     >sfive_i_flux_all,sfive_i_flux_low,sfive_i_flux_mid,
     >sfive_i_flux_high,
     >nfive_i_flux_all,nfive_i_flux_low,nfive_i_flux_mid,
     >nfive_i_flux_high)

c     THIS IS WHERE YOU CAN SET YOUR OWN INPUTS...DON'T SCREW UP KP!
c     choose_kp=6 IS EQUIVALENT TO Kp=2-

c     LOOP AROUND THE CALL_CALC ROUTINE WITH YOUR OWN INPUTS AND DUMP
c     YOUR OWN OUTPUTS IF REQUIRED
      choose_en=2300.D0
      choose_lt=22.2D0
      choose_kp=6

c     PERFORM THE INTERPOLATION AND RETURN THE VALUES
      CALL CALC(choose_kp, choose_en, choose_lt,
     >mean_e_flux_all,mean_e_flux_low,mean_e_flux_mid,mean_e_flux_high,
     > std_e_flux_all, std_e_flux_low, std_e_flux_mid, std_e_flux_high,
     >five_e_flux_all,five_e_flux_low,five_e_flux_mid,five_e_flux_high,
     >tfive_e_flux_all,tfive_e_flux_low,tfive_e_flux_mid,
     >tfive_e_flux_high,
     > med_e_flux_all, med_e_flux_low, med_e_flux_mid, med_e_flux_high,
     >sfive_e_flux_all,sfive_e_flux_low,sfive_e_flux_mid,
     >sfive_e_flux_high,
     >nfive_e_flux_all,nfive_e_flux_low,nfive_e_flux_mid,
     >nfive_e_flux_high,
     >mean_i_flux_all,mean_i_flux_low,mean_i_flux_mid,mean_i_flux_high,
     > std_i_flux_all, std_i_flux_low, std_i_flux_mid, std_i_flux_high,
     >five_i_flux_all,five_i_flux_low,five_i_flux_mid,five_i_flux_high,
     >tfive_i_flux_all,tfive_i_flux_low,tfive_i_flux_mid,
     >tfive_i_flux_high,
     > med_i_flux_all, med_i_flux_low, med_i_flux_mid, med_i_flux_high,
     >sfive_i_flux_all,sfive_i_flux_low,sfive_i_flux_mid,
     >sfive_i_flux_high,
     >nfive_i_flux_all,nfive_i_flux_low,nfive_i_flux_mid,
     >nfive_i_flux_high,
     >final_e_flux_all, final_e_flux_low,
     >final_e_flux_mid,final_e_flux_high,
     >final_i_flux_all, final_i_flux_low, 
     >final_i_flux_mid,final_i_flux_high)

      WRITE(6,*) ''
      WRITE(6,*) '===================================================='
      WRITE(6,30)'    CHOSEN KP =',str_kp(choose_kp),'(INDEX=', 
     > kp_array(choose_kp),')'
      WRITE(6,40)'CHOSEN ENERGY =',choose_en, ' eV'
      WRITE(6,50)'    CHOSEN LT =',choose_lt, ' LT'
      WRITE(6,*) '===================================================='
      WRITE(6,*) ''

c     PRINT OUT ELECTRON VALUES TO SCREEN
      WRITE(6,*) '*** ELECTRONS ***'
      WRITE(6,20) head_arr(1), head_arr(2), head_arr(3), head_arr(4)
      WRITE(6,10) str_arr(1), final_e_flux_all(1),final_e_flux_low(1),
     >       final_e_flux_mid(1),final_e_flux_high(1)
      WRITE(6,10) str_arr(2), final_e_flux_all(2),final_e_flux_low(2),
     >       final_e_flux_mid(2),final_e_flux_high(2)
      WRITE(6,10) str_arr(3), final_e_flux_all(3),final_e_flux_low(3),
     >       final_e_flux_mid(3),final_e_flux_high(3)
      WRITE(6,10) str_arr(4), final_e_flux_all(4),final_e_flux_low(4),
     >       final_e_flux_mid(4),final_e_flux_high(4)
      WRITE(6,10) str_arr(5), final_e_flux_all(5),final_e_flux_low(5),
     >       final_e_flux_mid(5),final_e_flux_high(5)
      WRITE(6,10) str_arr(6), final_e_flux_all(6),final_e_flux_low(6),
     >       final_e_flux_mid(6),final_e_flux_high(6)
      WRITE(6,10) str_arr(7), final_e_flux_all(7),final_e_flux_low(7),
     >       final_e_flux_mid(7),final_e_flux_high(7)
      WRITE(6,*) ''

c     PRINT OUT ION VALUES
      WRITE(6,*) '*** IONS ***'
      WRITE(6,20) head_arr(1), head_arr(2), head_arr(3), head_arr(4)
      WRITE(6,10) str_arr(1), final_i_flux_all(1),final_i_flux_low(1),
     >       final_i_flux_mid(1),final_i_flux_high(1)
      WRITE(6,10) str_arr(2), final_i_flux_all(2),final_i_flux_low(2),
     >       final_i_flux_mid(2),final_i_flux_high(2)
      WRITE(6,10) str_arr(3), final_i_flux_all(3),final_i_flux_low(3),
     >       final_i_flux_mid(3),final_i_flux_high(3)
      WRITE(6,10) str_arr(4), final_i_flux_all(4),final_i_flux_low(4),
     >       final_i_flux_mid(4),final_i_flux_high(4)
      WRITE(6,10) str_arr(5), final_i_flux_all(5),final_i_flux_low(5),
     >       final_i_flux_mid(5),final_i_flux_high(5)
      WRITE(6,10) str_arr(6), final_i_flux_all(6),final_i_flux_low(6),
     >       final_i_flux_mid(6),final_i_flux_high(6)
      WRITE(6,10) str_arr(7), final_i_flux_all(7),final_i_flux_low(7),
     >       final_i_flux_mid(7),final_i_flux_high(7)

c     PROGRAM COMPLETE
      WRITE(6,*) ''
      WRITE(6,*) '-------------FLUX MODEL COMPLETE-------------'
      WRITE(6,*) ''

 10   FORMAT(A16, 4F18.6)
 20   FORMAT(17x,  4A18)
 30   FORMAT(A15,A3,2x,A7,I2,A1)
 40   FORMAT(A15,F10.3,A4)
 50   FORMAT(A15,F6.2,A3)
      STOP
 
      END
c=======================END FLUX MODEL===================================


